@@ -30,12 +30,14 @@
 #include <fstream>
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
 #include <climits>
 #include <assert.h>
+#include <sqlite3.h>
 
 using namespace std;
 
-int check_bankData_lookup(unsigned long tries = 5)
+int check_bankData_lookup(const std::string &filename, unsigned long tries = 5)
 {
     unsigned long nr_found = 0,
                   nr_notfound = 0,
@@ -45,7 +47,7 @@ int check_bankData_lookup(unsigned long tries = 5)
     bool found;
     std::string bankId;
 
-    AccountNumberCheck checker;
+    AccountNumberCheck checker(filename);
 
     for (unsigned long i = 0; i < tries; i++) {
         /*
@@ -80,36 +82,94 @@ int check_bankData_lookup(unsigned long tries = 5)
     return 0;
 }
 
+static int test_check(void *objPtr, int argc, char **argv, char **azColName)
+{
+    AccountNumberCheck *checker = (AccountNumberCheck *)objPtr;
+    char *blz = argv[1];
+    char *method = argv[3];
+    char *name = argv[4];
+    char *location = argv[5];
+    std::string kto;
+
+    unsigned long rand_kto = rand() % 9000000000 + 1000000000;
+    kto = std::to_string(rand_kto);
+    AccountNumberCheck::Result res;
+
+    res = checker->check(blz, kto);
+    std::cout << "Result of " << blz << ";" << kto << ";" << method << ";" << name
+              << ": " <<checker->resultToString(res) << std::endl;
+    return 0;
+}
+
 int check_testkontos(const std::string &filename)
 {
     assert(filename.size() > 0);
     AccountNumberCheck checker(filename);
-    AccountNumberCheck::Result res;
-    char blz[20], kto[20], method[200], info[200];
-    FILE *istr = fopen(filename.c_str(), "r");
-    while (fscanf(istr, "%20[^;];%20[^;];%200[^;];%200[^;\n]\n", blz, kto, method, info) > 0)
-    {
-        res = checker.check(blz, kto);
-        std::cout << "Result of " << blz << ";" << kto << ";" << method << ";" << info << ": "
-                  <<checker.resultToString(res) << std::endl;
+
+    sqlite3 *db;
+    int retcode;
+    char *zErrorMsg;
+    retcode = sqlite3_open(filename.c_str(), &db);
+    if (retcode) {
+        std::cerr << "Cannot open database: " << sqlite3_errmsg(db)
+                  << std::endl;
+        exit(1);
     }
-    fclose(istr);
+
+    std::string sql;
+    sql = "SELECT country,bankcode,bic,method,name,location,date(valid_upto) FROM 'institutions'";
+    retcode = sqlite3_exec(db, sql.c_str(), &test_check, &checker, &zErrorMsg);
+    if (retcode) {
+        std::cout <<zErrorMsg << std::endl;
+        exit(1);
+    }
+    sqlite3_close(db);
     return 0;
 }
 
+void printUsage()
+{
+    std::cout << "Usage:\n"
+              << "benchmark [<options>] tries \n"
+              << "  --help, -h:             This help message\n"
+              << "  --version, -v:          Print the version number and exit.\n"
+              << "  --file=<bankdata-file>: Use this specified file as database\n"
+    ;
+}
+
 int main(int argc, char **argv)
 {
     unsigned long tries = 5;
-    if (argc > 1) {
-        tries = atol(argv[1]);
+    if (argc < 2) {
+        printUsage();
+        return 1;
+    }
+    if (argc == 2 && (strcmp(argv[1], "-h") == 0 || strcmp(argv[1], "--help") == 0)) {
+        printUsage();
+        return 1;
+    } else if (argc == 2 && (strcmp(argv[1], "-v") == 0 || strcmp(argv[1], "--version") == 0)) {
+        cout << VERSION << endl;
+        return 0;
     }
-    check_bankData_lookup(tries);
 
-    std::string filename;
-    if (argc > 2) {
-        filename = argv[2];
+    const char *arg = "--file";
+    std::string bankdataFile;
+    if (argc > 1 && strstr(argv[1], arg)) {
+        bankdataFile = argv[1]+strlen(arg)+1;
+        if (argc > 2) {
+            tries = atol(argv[2]);
+        }
+    } else if (argc == 2 && *argv[1] != '-') {
+        tries = atol(argv[1]);
+    } else {
+        printUsage();
+        return 1;
     }
-    check_testkontos(filename);
+    srand(time(0));
+
+    check_bankData_lookup(bankdataFile, tries);
+
+    check_testkontos(bankdataFile);
 
     return 0;
 }
