@@ -51,6 +51,7 @@ string bankdataFile = "";
 string method = "";
 string infile = "";
 string outfile = "";
+string country = "";
 bool justReturnCode = false;
 int nextArg = 1;
 Outformat outformat = MULTILINE;
@@ -59,7 +60,7 @@ void printUsage()
 {
     cout << "Usage:" << endl;
     cout << PACKAGE
-         << " [--returncode] [--file=datafile] <bank-id> <account-id>\n"
+         << " [--returncode] [--file=datafile] <country-code> <bank-id> <account-id>\n"
          << "  --help, -h:             This help message\n"
          << "  --version, -v:          Print the version number and exit.\n"
          << "  --bankdata-path:        Print the path where bankdata files are expected, and exit.\n"
@@ -83,7 +84,7 @@ void printUsage()
     exit(1);
 }
 
-void checkArg(string argument)
+void checkArg(string argument, int i)
 {
     string argn1 = "--method=";
     string argn2 = "--outformat=";
@@ -91,6 +92,10 @@ void checkArg(string argument)
     string argn4 = "--infile=";
     string argn5 = "--outfile=";
 
+    if (i == 1 && (argument == "DE" || argument == "NL" || argument == "CH")) {
+        country = argument;
+        return;
+    }
     if (argument == "--version" || argument == "-V") {
         cout << PACKAGE " " VERSION << endl;
         exit(0);
@@ -121,7 +126,7 @@ void checkArg(string argument)
         } else {
             std::cerr
                 << PACKAGE
-            ": The parameter --outformat has an unknown value. Allowed values are \"multiline\" and \"oneline\"."
+                ": The parameter --outformat has an unknown value. Allowed values are \"multiline\" and \"oneline\"."
                 << endl;
         }
 
@@ -152,18 +157,21 @@ void checkArg(string argument)
 int main(int argc, char **argv)
 {
     for (int i = 1; i < argc; i++) {
-        checkArg(argv[i]);
+        checkArg(argv[i], i);
     }
 
     if (argc - nextArg < 2) {
         printUsage();
     }
 
+    if (country.empty()) {
+        country = "DE";
+    }
     // Can we open the specified bankdata file? If not, use the
     // installed data file
     AccountNumberCheck *check_ptr = 0;
     if (bankdataFile.empty()) {
-        check_ptr = new AccountNumberCheck();
+        check_ptr = AccountNumberCheck::createChecker(country);
 //       time_t mydate = time(NULL);//-24*3600*30*9;
 //       if (!check_ptr->loadDataForDate(mydate))
 //       {
@@ -174,15 +182,15 @@ int main(int argc, char **argv)
         // No need to load the data file a second time - the default
         // constructor already loads the most suitable file for today.
     } else {
-        ifstream test(bankdataFile.c_str());
-        if (test.fail()) {
+        if (AccountNumberCheck::isValidDatabase(bankdataFile)) {
+            check_ptr = new AccountNumberCheck(bankdataFile);
+        } else {
             std::cerr << PACKAGE ": Warning: Specified bankdata file '"
                       << bankdataFile
-                      << "' could not be opened. Trying default file."
+                      << "' is not a SQLite database. Trying default database.\n"
                       << std::endl;
+
             check_ptr = new AccountNumberCheck();
-        } else {
-            check_ptr = new AccountNumberCheck(bankdataFile);
         }
     }
     assert(check_ptr);
@@ -258,8 +266,9 @@ int main(int argc, char **argv)
             case MULTILINE:
                 cout << "Bank: "
                      << (found
-                    ? ("'" + bankData.bankName + "' at '" + bankData.location + "'")
-                    : "<unknown>") << " (" << bankId << ")" << endl
+                    ? ("'" + bankData.bankName + "'"
+                       +(!bankData.location.empty() ? (" at '" + bankData.location + "'") : "")
+                       ) : "<unknown>") << " (" << bankId << ")" << endl
                      << "Account: " << accountId << endl
                      << "Result is: " << "(" << finalresult << ") "
                      << AccountNumberCheck::resultToString(finalresult) << endl
