@@ -4,7 +4,7 @@
 int main(int argc, char **argv)
 {
     bool successful = true;
-    char *givenpath = getenv("BANKDATA_SRCPATH");
+    char *givenpath = getenv("XDG_DATA_DIRS");
     std::string filename
         = givenpath ? std::string(givenpath)+"/ibandata.txt" : "";
     // std::cout << "using " << filename << std::endl;
