@@ -365,11 +365,11 @@ AccountNumberCheck::Result method_27(const int *account, int *weight)
 {
     int transform[6][10] =
     { {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // zero for non-transform [0][x]
-      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6}, // first transform line   [1][x]
-      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4}, // second transform line  [2][x]
-      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3}, // third transform line   [3][x]
-      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}, // fourth transform line  [4][x]
-      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} }; // which transform line
+      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6},   // first transform line   [1][x]
+      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4},   // second transform line  [2][x]
+      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3},   // third transform line   [3][x]
+      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9},   // fourth transform line  [4][x]
+      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} };   // which transform line
 
     if (atol(array2Number(account).c_str()) < 1000000000) {
         number2Array("2121212120", weight);
@@ -392,11 +392,11 @@ AccountNumberCheck::Result method_29(const int *account, int *weight)
 {
     int transform[6][10] =
     { {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // zero for non-transform [0][x]
-      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6}, // first transform line   [1][x]
-      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4}, // second transform line  [2][x]
-      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3}, // third transform line   [3][x]
-      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}, // fourth transform line  [4][x]
-      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} }; // which transform line
+      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6},   // first transform line   [1][x]
+      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4},   // second transform line  [2][x]
+      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3},   // third transform line   [3][x]
+      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9},   // fourth transform line  [4][x]
+      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} };   // which transform line
 
     return algo07(account, transform);
 }
@@ -530,15 +530,15 @@ AccountNumberCheck::Result method_44(const int *account, int *weight)
     return algo01(11, weight, false, 10, account);
 }
 
+// method 45 was extended for due-date 2018-06-04
 AccountNumberCheck::Result method_45(const int *account, int *weight)
 {
-    // some numbers do not have a checksum
-    if (0 == account[0] || 1 == account[4]) {
+    // accountIDs starting with "0" or "48" do not have a check digit
+    // accountIDs that have "1" as the fifth digit do not have a check digit
+    if (0 == account[0] || 1 == account[4] || (4 == account[0] && 8 == account[1])) {
         return AccountNumberCheck::OK;
     }
-
-    number2Array("2121212120", weight);
-    return algo01(10, weight, true, 10, account);
+    return method_00(account, weight);
 }
 
 AccountNumberCheck::Result method_46(const int *account, int *weight)
@@ -627,8 +627,7 @@ AccountNumberCheck::Result method_51(const int *account, int *weight)
     return algo01(7, weight, false, 10, account); // Method D
 }
 
-AccountNumberCheck::Result method_52(const int *account, int *weight, const std::string &accountId,
-                                     const std::string &bankId)
+AccountNumberCheck::Result method_52(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
 {
     // Changed by Jens Gecius, only 10 digit accountIDs starting with "9",
     // *NOT* validated
@@ -642,8 +641,7 @@ AccountNumberCheck::Result method_52(const int *account, int *weight, const std:
 }
 
 // Added by Jens Gecius, *NOT* validated
-AccountNumberCheck::Result method_53(const int *account, int *weight, const std::string &accountId,
-                                     const std::string &bankId)
+AccountNumberCheck::Result method_53(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
 {
     // Changed by Jens Gecius, only 10 digit accountIDs starting with "9",
     // *NOT* validated
@@ -893,11 +891,11 @@ AccountNumberCheck::Result method_69(const int *account, int *weight)
 {
     int transform[6][10] =
     { {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}, // zero for non-transform [0][x]
-      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6}, // first transform line   [1][x]
-      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4}, // second transform line  [2][x]
-      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3}, // third transform line   [3][x]
-      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9}, // fourth transform line  [4][x]
-      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} }; // which transform line
+      {0, 1, 5, 9, 3, 7, 4, 8, 2, 6},   // first transform line   [1][x]
+      {0, 1, 7, 6, 9, 8, 3, 2, 5, 4},   // second transform line  [2][x]
+      {0, 1, 8, 4, 6, 2, 9, 5, 7, 3},   // third transform line   [3][x]
+      {0, 1, 2, 3, 4, 5, 6, 7, 8, 9},   // fourth transform line  [4][x]
+      {1, 4, 3, 2, 1, 4, 3, 2, 1, 0} };   // which transform line
 
     long int lastnine = atol(array2Number(account).substr(1).c_str());
     if (account[0] == 9 && lastnine >= 300000000 && lastnine <= 399999999) {
@@ -1644,13 +1642,13 @@ AccountNumberCheck::Result method_B0(const int *account, int *weight)
 // Checked with Bundesbank-Testnumbers
 AccountNumberCheck::Result method_B1(const int *account, int *weight)
 {
-    number2Array("1371371370", weight);
-    if (AccountNumberCheck::OK == algo01(10, weight, false, 10, account)) {
-        return AccountNumberCheck::OK; // variant 1 (method "05")
+    if (AccountNumberCheck::OK == method_05(account, weight)) {
+        return AccountNumberCheck::OK; // variant 1
     }
-    number2Array("1731731730", weight);
-    return algo01(10, weight, false, 10, account);
-    // variant 2 (method "01")
+    if (AccountNumberCheck::OK == method_01(account, weight)) {
+        return AccountNumberCheck::OK; // variant 2
+    }
+    return method_00(account, weight); //variant 3, extended for due-date 2017-06-05
 }
 
 // Added by Jens Gecius (new Method as of September 6th, 2004)
@@ -1712,8 +1710,7 @@ AccountNumberCheck::Result method_B5(const int *account, int *weight)
 // B6, B7 and B8 Added by Alexander Kurz
 // new method as of September 5th, 2005, checked with Bundesbank-Testnumbers
 // method B6 was last changed for due-date September, 5 2011
-AccountNumberCheck::Result method_B6(const int *account, int *weight, const std::string &accountId,
-                                     const std::string &bankId)
+AccountNumberCheck::Result method_B6(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
 {
     if (account[0] > 0 && account[0] <= 9) {
         return method_20(account, weight); // variant 1
@@ -1788,8 +1785,7 @@ AccountNumberCheck::Result method_B9(const int *account, int *weight)
     return AccountNumberCheck::ERROR;
 }
 
-AccountNumberCheck::Result method_C0(const int *account, int *weight, const std::string &accountId,
-                                     const std::string &bankId)
+AccountNumberCheck::Result method_C0(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
 {
     if (account[0] == 0 && account[1] == 0 && account[2] > 0) {
         if (AccountNumberCheck::OK == method_52(account, weight, accountId, bankId)) { // variant 1
@@ -1824,7 +1820,10 @@ AccountNumberCheck::Result method_C2(const int *account, int *weight)
     if (AccountNumberCheck::OK == method_22(account, weight)) { // variant 1
         return AccountNumberCheck::OK;
     }
-    return method_00(account, weight);   // variant 2
+    if (AccountNumberCheck::OK == method_00(account, weight)) { // variant 2
+        return AccountNumberCheck::OK;
+    }
+    return method_04(account, weight); // variant 3, extended for due-date 2017-09-04
 }
 
 // new method C3 and C4 valid from March 5th, 2007, checked with Bundesbank-Testnumbers
@@ -2190,14 +2189,59 @@ AccountNumberCheck::Result method_E3(const int *account, int *weight)
     return method_21(account, weight);               // variant 2
 }
 
+// new method E4 valid from 2017-06-05
+// checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_E4(const int *account, int *weight)
+{
+    if (AccountNumberCheck::OK == method_02(account, weight)) { // variant 1
+        return AccountNumberCheck::OK;
+    }
+    return method_00(account, weight); // variant 2
+}
+
+AccountNumberCheck::Result method_4ea_10en(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
+{
+    if (bankId.size() != 4) {
+        return AccountNumberCheck::BANK_NOT_KNOWN;
+    }
+    for (int i = 0; i < 4; ++i) {
+        if (std::isalpha(bankId[i]) == false) {
+            return AccountNumberCheck::BANK_NOT_KNOWN;
+        }
+    }
+
+    if (accountId.size() != 10) {
+        return AccountNumberCheck::ERROR;
+    }
+
+    return AccountNumberCheck::OK;
+}
+
+AccountNumberCheck::Result method_5en_12ec(const int *account, int *weight, const std::string &accountId, const std::string &bankId)
+{
+    if (bankId.size() != 5) {
+        return AccountNumberCheck::BANK_NOT_KNOWN;
+    }
+
+    if (accountId.size() != 12) {
+        return AccountNumberCheck::ERROR;
+    }
+
+    for (int i = 0; i < 12; ++i) {
+        if (std::isalnum(accountId[i]) == false) {
+            return AccountNumberCheck::ERROR;
+        }
+    }
+
+    return AccountNumberCheck::OK;
+}
+
 struct method_func_s {
     const char *str;
     AccountNumberCheck::MethodFunc func;
 };
 const struct method_func_s cb_funcs[] = {
-    {
-        "00", method_00
-    },
+    { "00", method_00},
     { "01", method_01},
     { "02", method_02},
     { "03", method_03},
@@ -2337,6 +2381,7 @@ const struct method_func_s cb_funcs[] = {
     { "E1", method_E1},
     { "E2", method_E2},
     { "E3", method_E3},
+    { "E4", method_E4},
     { 0, 0} // Important: The array has to end with the {0,0} entry,
     // otherwise initMethodMap() will runaway and crash!
 };
@@ -2353,12 +2398,12 @@ struct method_funcLong_s {
     AccountNumberCheck::MethodFuncLong func;
 };
 const struct method_funcLong_s cb_funcs_long[] = {
-    {
-        "52", method_52
-    },
+    { "52", method_52},
     { "53", method_53},
     { "B6", method_B6},
     { "C0", method_C0},
+    { "4!a10!n", method_4ea_10en},
+    { "5!n12!c", method_5en_12ec},
     { 0, 0} // Important: The array has to end with the {0,0} entry,
             // otherwise initMethodMap() will runaway and crash!
 };
